/*
    LiveURL's - a Firefox extension that lets you create links to
    specific content on web pages

    Copyright (C) <2006>  Infosys Technologies Ltd.
    Authors : Natarajan Kannan (Natarajan_Kannan@infosys.com)
              Toufeeq Hussain (Toufeeq_Hussain@infosys.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


var commonUtils = {

    liveNodes : [],

    /* delimiter used to separate parameters in fragment ID, using '+'
     * has advantages.  It does not change on url-encoding, no other
     * part of the fragment ID generated by liveurls will have a +
     * except for the start string to identify which we use the length
     * and not the delimiter.
     */
    delim : "+",
    
    /* common utility functions used by both "Search" and "DOM" based
     * approaches 
     */

    /* does nothing */
    cleanStrings: function(str)
    {
        /* TODO: Wanted to do some processing here, but realized it is
         * better left alone, i.e. return the given string as it is
         */

        // return str.replace(/[\n\t\r ]+/g, " ");
        return str;
    },

    /* Adds a given range to the current window selection */
    selectRange: function(range)
    {
        var selection = content.window.getSelection();
        if(selection.rangeCount > 0)
            selection.removeAllRanges();
        selection.addRange(range);
    },

    /* Displays only the given range, stripping out all the other DOM nodes
     */
    displayRange: function(range)
    {
        var fragment = range.cloneContents();
        var body =
            content.document.getElementsByTagName("body").item(0);

        body.parentNode.replaceChild(fragment, body);
    },

    /* Given a range, it highlights all the text nodes within the range
     * The following function has been generously re-used from the
     * Scrapbook extension for Firefox
     * http://amb.vis.ne.jp/mozilla/scrapbook/
     */
    highlightRange: function(range)
    {
        var startNode = range.startContainer;
        var startOffset = range.startOffset;
        var endNode = range.endContainer;
        var endOffset = range.endOffset;
        var retNode = null;

        var sameNode = (startNode == endNode);

        /* If the startNode and the endNode are not same, we need
         * to walk through nodes between startNode and endNode
         */

        // if(!sameNode || !this.isTextNode(startNode)) 
        if(!sameNode) {
            var nodeWalker =
            content.document.createTreeWalker(range.commonAncestorContainer,
                NodeFilter.SHOW_TEXT, this.acceptNode, false);

            nodeWalker.currentNode = startNode;

            for(var txtNode = nodeWalker.nextNode(); 
                txtNode && txtNode != endNode; 
                txtNode = nodeWalker.nextNode()) {
                nodeWalker.currentNode = 
                    this.wrapTextNodeWithSpan(content.document, txtNode);
                retNode = nodeWalker.currentNode.parentNode.nextSibling;
            }
        }

        /* split the endNode if the selection ends in the middle of
         * the end node
         */
        if(this.isTextNode(endNode)) {
            endNode.splitText(endOffset);
        }

        if(!sameNode && this.isTextNode(endNode)) {
            retNode = this.wrapTextNodeWithSpan(content.document, endNode);
            retNode = retNode.parentNode.nextSibling;
        }

        /* handle segmented startNodes,
         */
        if(this.isTextNode(startNode)) {
            var secondHalf = startNode.splitText(startOffset);
            var tmpNode = 
                this.wrapTextNodeWithSpan(content.document, secondHalf);
            if(sameNode) {
                retNode = tmpNode.parentNode.nextSibling;
            }
        }

        /* we invalidate the liveNodes array, if anyone wants, they
         * recalculate using XPath
         */
        this.liveNodes = [];

        return retNode;
    },


    /* To test if a DOM node is a true TEXT node 
     */
    isTextNode: function(aNode) 
    {
        return ((aNode.nodeType == aNode.TEXT_NODE)
        && (aNode.nodeType != aNode.CDATA_SECTION_NODE)
        && (aNode.nodeType != aNode.PROCESSING_INSTRUCTION_NODE));
    },

    /* To test if a DOM node is a TEXT type node 
     */
    isTextTypeNode: function(aNode) 
    {
        return ((aNode.nodeType == aNode.TEXT_NODE)
            || (aNode.nodeType == aNode.CDATA_SECTION_NODE)
            || (aNode.nodeType == aNode.PROCESSING_INSTRUCTION_NODE));
    },


    /* Wraps a given node with a highlight span node
     * The following function has been generously re-used from the
     * Scrapbook extension for Firefox
     * http://amb.vis.ne.jp/mozilla/scrapbook/
     */
    wrapTextNodeWithSpan: function(aDoc, aTextNode)
    {
        var clonedTextNode = aTextNode.cloneNode( false );
        var nodeParent   = aTextNode.parentNode;
        var spanNode = aDoc.createElement("span");
        spanNode.setAttribute("style", "background-color: rgb(255, 255, 0);");
        spanNode.setAttribute("class", 'livetext');

        spanNode.appendChild( clonedTextNode );
        nodeParent.replaceChild(spanNode, aTextNode);
        return clonedTextNode;
    },

    /* Clears all highlighted nodes in the page
     */
    clearHighlight: function()
    {
        if(!this.liveNodes.length > 0) {
            this.liveNodes = content.document.evaluate(
                "//span[@class='livetext']",
                content.document,
                null,
                XPathResult.ORDERED_NODE_SNAPSHOT_TYPE,
                null);
        }
        var savedXOffset;
        var savedYOffset;
        
        if(this.liveNodes.snapshotLength > 0) {
            savedXOffset = content.window.pageXOffset;
            savedYOffset = content.window.pageYOffset;
        }

        for(var i = 0; i < this.liveNodes.snapshotLength; i++) {
            var liveNode = this.liveNodes.snapshotItem(i);
            var parent = liveNode.parentNode;
            parent.replaceChild(liveNode.childNodes[0], liveNode)
            parent.normalize();
        }

        if(this.liveNodes.snapshotLength > 0) {
            content.document.location.hash = "#";
            content.window.scroll(savedXOffset, savedYOffset);
        }

        /* we invalidate the liveNodes array, if anyone wants it, they
         * recalculate it
         */
        this.liveNodes = [];
    },

    isLivePage: function()
    {
        if(!this.liveNodes.length > 0) {
            this.liveNodes = content.document.evaluate(
                "//span[@class='livetext']",
                content.document,
                null,
                XPathResult.ORDERED_NODE_SNAPSHOT_TYPE,
                null);
        }

        if(this.liveNodes.snapshotLength > 0) {
            return true;
        } else {
            return false;
        }
    },

    focusLiveText: function(instance)
    {
        if(!this.liveNodes.length > 0) {
            this.liveNodes = content.document.evaluate(
                "//span[@class='livetext']",
                content.document,
                null,
                XPathResult.ORDERED_NODE_SNAPSHOT_TYPE,
                null);
        }

        if(this.liveNodes.snapshotLength > instance) {
            this.liveNodes.snapshotItem(instance).scrollIntoView();
        }
    },

    /* debugging function, see output in Javascript Console window of
     * Firefox
     */
    output: function(text) 
    {
        var consoleService =  
            Components.classes['@mozilla.org/consoleservice;1'].
            getService(Components.interfaces.nsIConsoleService);
        consoleService.logStringMessage(text);
    },

    handleLiveURLMenu: function(e)
    {
        if (this.isLivePage()) {
            document.getElementById("context-liveurls-highlight").hidden 
                = false;
        } else {
            document.getElementById("context-liveurls-highlight").hidden 
                = true;
        }   
    },

     
    /* Checks for text nodes with only white space characters and
     * rejects them.
     * The following function has been generously re-used from the
     * Scrapbook extension for Firefox
     * http://amb.vis.ne.jp/mozilla/scrapbook/
     */
    acceptNode: function(aNode)
    {
        /* If aNode is a TEXT_NODE and it contains only whitespace
         * characters, we reject it
         */
        if ( aNode.nodeType == aNode.TEXT_NODE
        && ! ( /[^\t\n\r ]/.test( aNode.nodeValue )) 
        ) {
            return NodeFilter.FILTER_REJECT;
        }

        return NodeFilter.FILTER_ACCEPT;
    },

    /* This function handles the failure of processing of a LiveURL.
     * It displays a message saying the LiveURL failed and clears the URL bar.
     */
    handleFailure: function()
    {
        if(!this.isLivePage()) {
            alert("LiveURL processing failed.  The indicated content \
            might have changed.  If you still feel this is a bug, please \
            report the bug at http://liveurls.mozdev.org");

            /* reset the fragment ID,  the live fragment ID isn't valid
             * any more
             */
            content.document.location.hash = "#";
        }
    },

    copyToClipboard: function(liveURL)
    {
        const gClipboardHelper = 
                Components.classes["@mozilla.org/widget/clipboardhelper;1"].
                getService(Components.interfaces.nsIClipboardHelper);
        gClipboardHelper.copyString(liveURL);
    },

};
